@@ -10,13 +10,12 @@ import csv
 import copy
 import json
 import pickle
-import typing
 import pathlib
+from typing import *
 
-import pymssql
-import pymysql
 import numpy as np
 import pandas as pd
+from sqlalchemy import create_engine
 from tensorflow.keras.preprocessing.sequence import pad_sequences
 
 from fern import setting
@@ -59,31 +58,28 @@ class FernDataFrame(pd.DataFrame):
             path where data save
         """
         common.check_path(path)
-        self.to_csv(path, index=False, encoding='utf-8-sig', quoting=csv.QUOTE_NONNUMERIC)
+        self.to_csv(path, index=True, encoding='utf-8-sig', quoting=csv.QUOTE_NONNUMERIC)
 
 
 class FernDownloader(object):
-    def __init__(self, host, user, password):
+    def __init__(self, url):
         """
         data downloader
 
         Parameters
         ----------
-        host : str
-                sql server host
-        user : str
-            user name
-        password : str
-            user password
-        """
-        self.data: typing.Optional[FernDataFrame] = None
-        self.host = host
-        self.user = user
-        self.password = password
+        url: str
+            The string form of the URL is dialect[+driver]://user:password@host/dbname[?key=value..]
 
-    def read_msssql(self, sql, index=None, drop=False):
-        conn = pymssql.connect(host=self.host, user=self.user, password=self.password, charset=r'utf8')
-        df = pd.read_sql(sql, conn)
+        Examples
+        --------
+        downloader = FernDownloader('mysql+pymysql://user:passwd@hostname/dbname?charset=utf8mb4')
+        """
+        self.engine = create_engine(url)
+        self.data: Optional[FernDataFrame] = None
+
+    def read_sql(self, sql, index=None, drop=False):
+        df = pd.read_sql(sql, self.engine)
         df = FernDataFrame(df)
 
         if drop:
@@ -92,22 +88,6 @@ class FernDownloader(object):
             self.data = df
         else:
             self.data = df.set_index(index)
-        conn.close()
-        return self
-
-    def read_mysql(self, sql, index=None, drop=False):
-        conn = pymysql.connect(host=self.host, user=self.user, password=self.password, charset=r'utf8')
-        df = pd.read_sql(sql, conn)
-        df = FernDataFrame(df)
-
-        if drop:
-            df = df.dropna()
-        if index is None:
-            self.data = df
-        else:
-            self.data = df.dropna().set_index(index)
-        conn.close()
-        return self
 
     def save(self, path):
         """save data frame into csv"""
@@ -117,22 +97,8 @@ class FernDownloader(object):
 class FernCleaner(object):
     """
     data cleaner
-
-    Parameters
-    ----------
-    stop_words : str, Path, optional
-        stop words path
-    cut_func : typing.Callable
-        a function to split sequence to word list
-    update_data : bool
-        To save the cleaned data to self.data. The default is True.
-    data_col : str
-        The column name of the input data
-    label_col : str
-        The column name of the output label
-    idx_col : str, optional
-        The index column name for source data. If not provided, the output columns will not contain index column.
     """
+
     def __init__(self,
                  stop_words=None,
                  cut_func=None,
@@ -140,7 +106,23 @@ class FernCleaner(object):
                  data_col='data',
                  label_col='label',
                  idx_col=None):
-        self.data: typing.Optional[FernDataFrame] = None
+        """
+        Parameters
+        ----------
+        stop_words : str, Path, optional
+            stop words path
+        cut_func : typing.Callable
+            a function to split sequence to word list. If you want to use user_words, please define here
+        update_data : bool
+            To save the cleaned data to self.data. The default is True.
+        data_col : str
+            The column name of the input data
+        label_col : str
+            The column name of the output label
+        idx_col : str, optional
+            The index column name for source data. If not provided, the output columns will not contain index column.
+        """
+        self.data: Optional[FernDataFrame] = None
         self.logger = setting.LOGGER
         self.stop_words = common.read_regex_words(stop_words)
         self.update_data = update_data
@@ -238,7 +220,7 @@ class FernCleaner(object):
         data = self.cut_func(string)
         words = []
         for da in data:
-            da = re.sub('[^a-zA-Z0-9\-_. ]', '', da)
+            da = re.sub('[^a-zA-Z0-9\-_.\u4e00-\u9fa5<> ]', '', da)  # delete all unimportant words
             if len(da) > 1 and da != ' ' and not self.is_stop_words(da):
                 words.append(da)
 
@@ -291,6 +273,7 @@ class FernCleaner(object):
         """
         data = pd.read_csv(path, index_col=self.idx_col)
         data[self.label_col] = data[self.label_col].map(eval)
+        data[self.data_col] = data[self.data_col].map(eval)
         self.data = data
 
 
@@ -304,7 +287,7 @@ class FernTransformer(object):
         Path to word library
     label_path : str
         Path to label data
-    output_shape : dict[str, int], list[int], tuple[int], optional
+    output_shape : Optional[Dict[str, Union[List[int], int]]]
         output shape of label for transforming label.
         you don't have to defined it if you don't use it to transform label
     min_len : int
@@ -324,9 +307,10 @@ class FernTransformer(object):
         Whether to delete the input data which the sentence length is greater than max_len or less than min_len
     """
     PREFIX = [
-        '<PAD>',    # 占位符
-        '<ST>',     # 开始字符
-        '<ED>'      # 终止字符
+        '<PAD>',  # 占位符
+        '<ST>',  # 开始字符
+        '<ED>',  # 终止字符
+        '<SEP>'  # 分割符号
     ]
 
     def __init__(self,
@@ -340,7 +324,7 @@ class FernTransformer(object):
                  data_col='data',
                  label_col='label',
                  filter_data=True):
-        self.data: typing.Optional[typing.Dict[str, typing.Union[np.ndarray, typing.Dict[str, np.ndarray]]]] = None
+        self.data: Optional[Dict[str, Union[np.ndarray, Dict[str, np.ndarray]]]] = None
 
         self.data_col = data_col
         self.label_col = label_col
@@ -380,10 +364,10 @@ class FernTransformer(object):
         data[self.label_col] = data[self.label_col].map(self.transform_label)
         data = data.dropna()
 
-        data_ = np.concatenate(data[self.data_col])
+        data_ = np.concatenate(data[self.data_col].to_list())
 
         labels = {}
-        label = pd.DataFrame(data[self.label_col].to_list())    # data.label_col 每一行都是字典
+        label = pd.DataFrame(data[self.label_col].to_list())  # data.label_col 每一行都是字典
         for col in label.columns:
             labels[col] = np.concatenate(label[col])
 
@@ -409,7 +393,7 @@ class FernTransformer(object):
             The transformed word id sequence
         """
         data = [self.word2id[word] for word in data if word in self.word2id]
-        data = pad_sequences([data], maxlen=self.max_len, padding='post')       # pad after each sequence
+        data = pad_sequences([data], maxlen=self.max_len, padding='post')  # pad after each sequence
         return data
 
     def transform_label(self, label):
@@ -424,7 +408,9 @@ class FernTransformer(object):
         Returns
         -------
         dict[str, np.ndarray]
-            The transformed label and make sure using float32
+            The transformed label.And Note that:
+             - Please make sure using float32
+             - Even the 2 category outputs An Array in a multi-category format
         """
         raise NotImplementedError
 
@@ -474,7 +460,7 @@ class FernTransformer(object):
             raise ValueError('No data is provided.')
 
         label_data = {}
-        df_label = pd.DataFrame(data[self.label_col].to_list())    # data.label_col每一行都是字典
+        df_label = pd.DataFrame(data[self.label_col].to_list())  # data.label_col每一行都是字典
         for col in df_label.columns:
             tmp = set()
             df_label[col].map(lambda x: tmp.update(x))
@@ -499,9 +485,7 @@ class FernTransformer(object):
             The word to id dictionary
         """
         if pathlib.Path(self.word_path).exists():
-            with open(self.word_path, 'r', encoding='utf-8') as f:
-                words = f.readlines()
-                words = [word.strip().lower() for word in words]
+            words = common.read_words(self.word_path)
         else:
             words = self.reload_word_library(data)
         word2id = dict(zip(words, range(len(words))))
@@ -530,10 +514,21 @@ class FernTransformer(object):
             raise ValueError('No data is provided.')
 
         words = []
-        _ = data[self.data_col].map(words.extend)
+        words_append = words.append
+
+        def _map_func(word_list):
+            for word in word_list:
+                word = word.strip().lower()
+                if word and word[0] != '#':
+                    words_append(word)
+
+        _ = data[self.data_col].map(_map_func)
         words = pd.Series(words).value_counts()
         words = words[words >= self.max_freq]
         words = list(words.index)
+
+        words = [word for word in words if word.upper() not in self.PREFIX]
+
         words = self.PREFIX + words
 
         data = '\n'.join(words)
@@ -555,6 +550,7 @@ class FernTransformer(object):
         pd.DataFrame
             the filtered data frame
         """
+
         def __filter_func(item):
             """
             to filter data
@@ -571,6 +567,7 @@ class FernTransformer(object):
             """
             item = [word for word in item if word in self.word2id]
             return self.min_len <= len(item) <= self.max_len
+
         return data[data[self.data_col].map(__filter_func)]
 
     def save(self, path):
@@ -604,8 +601,9 @@ class FernTransformer(object):
 
 class FernSplitter(object):
     """split data into train data and val data"""
+
     def __init__(self, rate_val, random_state=None):
-        self.data: typing.Optional[typing.Dict[str, typing.Union[np.ndarray, typing.Dict[str, np.ndarray]]]] = None
+        self.data: Optional[Dict[str, Union[np.ndarray, Dict[str, np.ndarray]]]] = None
 
         self.rate_val = rate_val
         self.random_state = random_state
@@ -709,7 +707,7 @@ class FernBalance(object):
         self.data_col = data_col
         self.label_col = label_col
         self.random_state = random_state
-        self.data: typing.Optional[typing.Dict[str, typing.Union[np.ndarray, typing.Dict[str, np.ndarray]]]] = None
+        self.data: Optional[Dict[str, Union[np.ndarray, Dict[str, np.ndarray]]]] = None
         self.logger = setting.LOGGER
 
     def balance(self, data):
@@ -769,7 +767,7 @@ class FernBalance(object):
         for col_idx, num in sorted(enumerate(count_label), key=lambda item: item[1]):
             row_idx = np.argwhere(label[:, col_idx] == 1).reshape(-1)
             row_idx = np.random.RandomState(self.random_state).permutation(row_idx)
-            if num >= target_num or not num:
+            if num >= target_num or num < 2:
                 continue
             repeat_times = int(target_num // num - 1)
             copy_num = int(target_num % num)
